@@ -24,9 +24,10 @@ from __future__ import unicode_literals
 import six
 import sys
 import time
+from collections import deque
 from functools import partial
-from requests.exceptions import HTTPError
 from timeit import default_timer as timer
+from praw.errors import HTTPException
 
 BACKOFF_START = 4  # Minimum number of seconds to sleep during errors
 KEEP_ITEMS = 128  # On each iteration only remember the first # items
@@ -56,7 +57,7 @@ def comment_stream(reddit_session, subreddit, limit=None, verbosity=1):
     """
     get_function = partial(reddit_session.get_comments,
                            six.text_type(subreddit))
-    return _stream_generator(get_function, reddit_session, limit, verbosity)
+    return _stream_generator(get_function, limit, verbosity)
 
 
 def submission_stream(reddit_session, subreddit, limit=None, verbosity=1):
@@ -88,11 +89,29 @@ def submission_stream(reddit_session, subreddit, limit=None, verbosity=1):
             limit = 1000
     if not hasattr(subreddit, 'reddit_session'):
         subreddit = reddit_session.get_subreddit(subreddit)
-    return _stream_generator(subreddit.get_new, reddit_session, limit,
-                             verbosity)
+    return _stream_generator(subreddit.get_new, limit, verbosity)
 
 
-def _stream_generator(get_function, reddit_session, limit=None, verbosity=1):
+def valid_redditors(redditors, sub):
+    """Return a verified list of valid Redditor instances.
+
+    :param redditors: A list comprised of Redditor instances and/or strings
+        that are to be verified as actual redditor accounts.
+    :param sub: A Subreddit instance that the authenticated account has
+        flair changing permission on.
+
+    Note: Flair will be unset for all valid redditors in `redditors` on the
+    subreddit `mod_sub`.
+
+    """
+    simplified = list(set(six.text_type(x).lower() for x in redditors))
+    return [sub.reddit_session.get_redditor(simplified[i], fetch=False)
+            for (i, resp) in enumerate(sub.set_flair_csv(
+                ({'user': x, 'flair_text': x} for x in simplified)))
+            if resp['ok']]
+
+
+def _stream_generator(get_function, limit=None, verbosity=1):
     def debug(msg, level):
         if verbosity >= level:
             sys.stderr.write(msg + '\n')
@@ -111,7 +130,7 @@ def _stream_generator(get_function, reddit_session, limit=None, verbosity=1):
         start = timer()
         try:
             i = None
-            params = {'count': count}
+            params = {'uniq': count}
             count = (count + 1) % 100
             if before:
                 params['before'] = before
@@ -131,7 +150,7 @@ def _stream_generator(get_function, reddit_session, limit=None, verbosity=1):
                     items.append(item)
                     processed += 1
                 if verbosity >= 1 and processed % 100 == 0:
-                    sys.stderr.write(' Items: {0}          \r'
+                    sys.stderr.write(' Items: {0}            \r'
                                      .format(processed))
                     sys.stderr.flush()
                 if i < KEEP_ITEMS:
@@ -143,7 +162,7 @@ def _stream_generator(get_function, reddit_session, limit=None, verbosity=1):
                     # old or no longer exist.
                     before = None
             backoff = BACKOFF_START
-        except HTTPError as exc:
+        except HTTPException as exc:
             sleep = (backoff, '{0}. Sleeping for {{0}} seconds.'.format(exc),
                      2)
             backoff *= 2
@@ -158,7 +177,7 @@ def _stream_generator(get_function, reddit_session, limit=None, verbosity=1):
             yield item
         # Sleep if necessary
         if sleep:
-            sleep_time, msg, msg_level = sleep
+            sleep_time, msg, msg_level = sleep  # pylint: disable=W0633
             debug(msg.format(sleep_time), msg_level)
             time.sleep(sleep_time)
 
@@ -174,12 +193,14 @@ def convert_id36_to_numeric_id(id36):
 def convert_numeric_id_to_id36(numeric_id):
     """Convert an integer into its base36 string representation.
 
-    This method has been cleaned up slightlyto improve readability. For more
+    This method has been cleaned up slightly to improve readability. For more
     info see:
 
     https://github.com/reddit/reddit/blob/master/r2/r2/lib/utils/_utils.pyx
-    http://www.reddit.com/r/redditdev/comments/n624n/submission_ids_question/
-    http://en.wikipedia.org/wiki/Base_36#Python_implementation
+
+    https://www.reddit.com/r/redditdev/comments/n624n/submission_ids_question/
+
+    https://en.wikipedia.org/wiki/Base36
     """
     # base36 allows negative numbers, but reddit does not
     if not isinstance(numeric_id, six.integer_types) or numeric_id < 0:
@@ -215,15 +236,14 @@ def flatten_tree(tree, nested_attr='replies', depth_first=False):
         rather than the default breadth-first manner.
 
     """
-    stack = tree[:]
+    stack = deque(tree)
+    extend = stack.extend if depth_first else stack.extendleft
     retval = []
     while stack:
-        item = stack.pop(0)
+        item = stack.popleft()
         nested = getattr(item, nested_attr, None)
-        if nested and depth_first:
-            stack.extend(nested)
-        elif nested:
-            stack[0:0] = nested
+        if nested:
+            extend(nested)
         retval.append(item)
     return retval
 
@@ -239,18 +259,20 @@ def normalize_url(url):
 
 class BoundedSet(object):
 
-    """A set with a maxmimum size that evicts the oldest items when necessary.
+    """A set with a maximum size that evicts the oldest items when necessary.
 
     This class does not implement the complete set interface.
 
     """
 
     def __init__(self, max_items):
+        """Construct an instance of the BoundedSet."""
         self.max_items = max_items
         self._fifo = []
         self._set = set()
 
     def __contains__(self, item):
+        """Test if the BoundedSet contains item."""
         return item in self._set
 
     def add(self, item):
