@@ -12,16 +12,25 @@
 # You should have received a copy of the GNU General Public License along with
 # PRAW.  If not, see <http://www.gnu.org/licenses/>.
 
-"""Internal helper functions."""
+"""Internal helper functions.
 
-from requests import Request
+The functions in this module are not to be relied upon by third-parties.
+
+"""
+
+from __future__ import print_function, unicode_literals
+import re
 import six
 import sys
+from requests import Request, codes, exceptions
 from requests.compat import urljoin
 from praw.decorators import restrict_access
-from praw.errors import (InvalidSubreddit, OAuthException,
-                         OAuthInsufficientScope, OAuthInvalidToken,
-                         RedirectException)
+from praw.errors import (HTTPException, Forbidden, NotFound, InvalidSubreddit,
+                         OAuthException, OAuthInsufficientScope,
+                         OAuthInvalidToken, RedirectException)
+
+
+RE_REDIRECT = re.compile('(rand(om|nsfw))|about/sticky')
 
 
 def _get_redditor_listing(subpath=''):
@@ -29,9 +38,11 @@ def _get_redditor_listing(subpath=''):
     def _listing(self, sort='new', time='all', *args, **kwargs):
         """Return a get_content generator for some RedditContentObject type.
 
-        :param sort: Specify the sort order of the results if applicable.
+        :param sort: Specify the sort order of the results if applicable
+            (one of ``'hot'``, ``'new'``, ``'top'``, ``'controversial'``).
         :param time: Specify the time-period to return submissions if
-            applicable.
+            applicable (one of ``'hour'``, ``'day'``, ``'week'``,
+            ``'month'``, ``'year'``, ``'all'``).
 
         The additional parameters are passed directly into
         :meth:`.get_content`. Note: the `url` parameter cannot be altered.
@@ -40,7 +51,7 @@ def _get_redditor_listing(subpath=''):
         kwargs.setdefault('params', {})
         kwargs['params'].setdefault('sort', sort)
         kwargs['params'].setdefault('t', time)
-        url = urljoin(self._url, subpath)  # pylint: disable-msg=W0212
+        url = urljoin(self._url, subpath)  # pylint: disable=W0212
         return self.reddit_session.get_content(url, *args, **kwargs)
     return _listing
 
@@ -59,7 +70,7 @@ def _get_sorter(subpath='', **defaults):
             kwargs['params'] = {}
         for key, value in six.iteritems(defaults):
             kwargs['params'].setdefault(key, value)
-        url = urljoin(self._url, subpath)  # pylint: disable-msg=W0212
+        url = urljoin(self._url, subpath)  # pylint: disable=W0212
         return self.reddit_session.get_content(url, *args, **kwargs)
     return _sorted
 
@@ -76,6 +87,12 @@ def _modify_relationship(relationship, unlink=False, is_sub=False):
 
     if relationship == 'friend':
         access = {'scope': None, 'login': True}
+    elif relationship == 'moderator':
+        access = {'scope': 'modothers'}
+    elif relationship in ['banned', 'contributor']:
+        access = {'scope': 'modcontributors'}
+    elif relationship in ['wikibanned', 'wikicontributor']:
+        access = {'scope': ['modcontributors', 'modwiki']}
     else:
         access = {'scope': None, 'mod': True}
 
@@ -97,45 +114,59 @@ def _modify_relationship(relationship, unlink=False, is_sub=False):
     return do_relationship
 
 
-def _prepare_request(reddit_session, url, params, data, auth, files):
+def _prepare_request(reddit_session, url, params, data, auth, files,
+                     method=None):
     """Return a requests Request object that can be "prepared"."""
     # Requests using OAuth for authorization must switch to using the oauth
     # domain.
     if getattr(reddit_session, '_use_oauth', False):
         headers = {'Authorization': 'bearer %s' % reddit_session.access_token}
         config = reddit_session.config
-        # pylint: disable-msg=W0212
-        for prefix in (config._site_url, config._ssl_url):
+        for prefix in (config.api_url, config.permalink_url):
             if url.startswith(prefix):
                 if config.log_requests >= 1:
-                    sys.stderr.write('substituting %s for %s in url\n'
-                                     % (config._oauth_url, prefix))
-                url = config._oauth_url + url[len(prefix):]
+                    sys.stderr.write('substituting {} for {} in url\n'
+                                     .format(config.oauth_url, prefix))
+                url = config.oauth_url + url[len(prefix):]
                 break
     else:
         headers = {}
     headers.update(reddit_session.http.headers)
+
+    if method:
+        pass
+    elif data or files:
+        method = 'POST'
+    else:
+        method = 'GET'
+
     # Log the request if logging is enabled
     if reddit_session.config.log_requests >= 1:
-        sys.stderr.write('retrieving: %s\n' % url)
+        sys.stderr.write('{0}: {1}\n'.format(method, url))
     if reddit_session.config.log_requests >= 2:
-        sys.stderr.write('params: %s\n' % (params or 'None'))
-        sys.stderr.write('data: %s\n' % (data or 'None'))
+        if params:
+            sys.stderr.write('params: {0}\n'.format(params))
+        if data:
+            sys.stderr.write('data: {0}\n'.format(data))
         if auth:
-            sys.stderr.write('auth: %s\n' % str(auth))
+            sys.stderr.write('auth: {0}\n'.format(auth))
     # Prepare request
-    request = Request(method='GET', url=url, headers=headers, params=params,
+    request = Request(method=method, url=url, headers=headers, params=params,
                       auth=auth, cookies=reddit_session.http.cookies)
-    if not data and not files:  # GET request
+    if method == 'GET':
         return request
     # Most POST requests require adding `api_type` and `uh` to the data.
     if data is True:
         data = {}
-    if not auth:
-        data.setdefault('api_type', 'json')
-        if reddit_session.modhash:
-            data.setdefault('uh', reddit_session.modhash)
-    request.method = 'POST'
+
+    if isinstance(data, dict):
+        if not auth:
+            data.setdefault('api_type', 'json')
+            if reddit_session.modhash:
+                data.setdefault('uh', reddit_session.modhash)
+    else:
+        request.headers.setdefault('Content-Type', 'application/json')
+
     request.data = data
     request.files = files
     return request
@@ -147,14 +178,14 @@ def _raise_redirect_exceptions(response):
     Raise exceptions if appropriate.
 
     """
-    if response.status_code != 302:
+    if response.status_code not in [301, 302, 307]:
         return None
     new_url = urljoin(response.url, response.headers['location'])
-    if 'reddits/search?q=' in new_url:  # Handle non-existent subreddit
+    if 'reddits/search' in new_url:  # Handle non-existent subreddit
         subreddit = new_url.rsplit('=', 1)[1]
         raise InvalidSubreddit('`{0}` is not a valid subreddit'
                                .format(subreddit))
-    elif 'random' not in response.url:
+    elif not RE_REDIRECT.search(response.url):
         raise RedirectException(response.url, new_url)
     return new_url
 
@@ -169,7 +200,16 @@ def _raise_response_exceptions(response):
             raise OAuthInvalidToken('invalid_token', response.url)
         else:
             raise OAuthException(msg, response.url)
-    response.raise_for_status()
+
+    if response.status_code == codes.forbidden:  # pylint: disable=E1101
+        raise Forbidden(_raw=response)
+    elif response.status_code == codes.not_found:  # pylint: disable=E1101
+        raise NotFound(_raw=response)
+    else:
+        try:
+            response.raise_for_status()  # These should all be directly mapped
+        except exceptions.HTTPError as exc:
+            raise HTTPException(_raw=exc.response)
 
 
 def _to_reddit_list(arg):
@@ -179,9 +219,8 @@ def _to_reddit_list(arg):
     representation of an object. Either given as a string or as an object that
     is then converted to its string representation.
     """
-    if (isinstance(arg, six.string_types)
-            or not (hasattr(arg, "__getitem__")
-                    or hasattr(arg, "__iter__"))):
+    if (isinstance(arg, six.string_types) or not (
+            hasattr(arg, "__getitem__") or hasattr(arg, "__iter__"))):
         return six.text_type(arg)
     else:
         return ','.join(six.text_type(a) for a in arg)
